C*****************************************************
C*** Library: KUSS_FORTRAN
C***  Module: DFOURT
C*****************************************************
        PROGRAM DFOURTX
C----------------------------------------------------------------------------
C  Author:   J. Lehar                                         Date: 13-JUL-87
C  Written for VAX FORTRAN 77   (May require some changes for other machines)
C  NOTE: This code was written for clarity, not for efficiency.
C        Some optimization may be desired for certain applications.
C----------------------------------------------------------------------------
C  Produces the dirty spectrum and the spectral window for a time series
C  contained in XFILE.  The time average and the data mean are automatically
C  removed (see paper).  The frequency sampling is determined by the user.
C              / freq. resolution = 1/T   ; T is the overall time interval
C   defaults: <  Max. frequency   = 1/2dt ; dt is the smallest time spacing
C              \ Points-per-beam  = 4
C  The dirty spectrum is written to DFILE, and the spectral window to WFILE.
C  NOTE: the filenames are limited to 10 characters.  This is so they can
C        be propagated to subsequent programs through the spec. file headers
C----------------------------------------------------------------------------
C  CALLS:     RDDATA       to read the time series data  \_ in READWRITE.FOR
C             WRSPEC       to write spectra              /
C             DFOUR        performs the discrete FT
C----------------------------------------------------------------------------
        PARAMETER (MAXN= 20000)           ! maximum # data samples
        PARAMETER (MAXM= 2000000)         ! maximum index for FT array

C  Declare arrays and some variables
        COMPLEX      D(0:MAXM),W(0:2*MAXM),DFOUR
        REAL         T(MAXN),X(MAXN),ONES(MAXN),F(0:2*MAXM)
        REAL         FREQ(MAXM),SPECR(MAXM),SPECI(MAXM),POWER(MAXM)
        REAL         PMAX, FMAX
        CHARACTER    XFILE*20,DFILE*20,WFILE*20,HEADER*80
        DATA ONES/MAXN*1.0/           ! fill the ONES array with ones

C  Get execution parameters and filenames
         write(*,*) "INPUT FILES & PARAMETERS:"
       WRITE(*,100) ' Time-series data file: '
        READ (*,*) XFILE
       WRITE(*,100) ' Frequency resolution (0=default): '
        READ (*,*) FRES
       WRITE(*,100) ' Maximum frequency (0=default): '
        READ (*,*) FMAX
       WRITE(*,100) ' Points-per-beam (0=default): '
        READ (*,*) PPB
       WRITE (*,*) 
	 WRITE (*,*) ' Please, wait '
       WRITE (*,*) 

C  Read the times T(1:N) and the data X(1:N) from XFILE
        N= MAXN                               ! defines max. N for RDDATA
        CALL RDDATA(N,T,X,XFILE,HEADER)
C       TYPE *,HEADER
C        TYPE *, N
C  Find the mean T,X
        TMEAN= 0.
        XMEAN= 0.
        DO I=1,N
           TMEAN= TMEAN+ T(I)
           XMEAN= XMEAN+ X(I)
        ENDDO
        TMEAN= TMEAN/N
        XMEAN= XMEAN/N
c        WRITE(6,*), 'MEAN' , TMEAN, XMEAN
C  subtract the time,data means from T,X
        DO I=1,N
           T(I)= T(I)-TMEAN
           X(I)= X(I)-XMEAN
        ENDDO
c        WRITE(6,*), 'SUB'

C  If parameters are supplied as zero, set default values
        SMIN= 1.E20                          !\--(larger than expected SEPs)
        DO I=2,N                           ! \
           IF(T(I)-T(I-1).EQ.0) GOTO 898
           SEP= T(I) - T(I-1)                !  > find min. time separation
898        IF(SEP.LT.SMIN) SMIN= SEP         ! /
        ENDDO                                !/
        SMAX= T(N)-T(1)                      !> max. time separation
c        WRITE(6,*), 'bsmax'
        IF(FRES.EQ.0.) FRES= 1./SMAX         ! frequency resolution
c        WRITE(6,*), 'asmax', FRES
        IF(FMAX.EQ.0.) FMAX= 1./(2.*SMIN)    ! max. frequency
c        WRITE(6,*), 'asmin'
        IF(PPB.EQ.0.) PPB= 4.                ! points-per-beam
C    ---confirm parameter values
C        TYPE *,'FRES,FMAX,PPB=',FRES,FMAX,PPB

C  set up the frequency array F(0:M)
        dF= FRES/PPB                   ! frequency increment
        M= INT(FMAX/dF)                ! maximum freq. element (for D)
C        type *, 'M=',M
        DO J=0,2*M
           F(J)= dF*J
        ENDDO

C  calculate the dirty spectrum D(0:M) and the spec. window W(0:2M)
C        TYPE *,'Computing the dirty spectrum...'
        DO J= 0,M
           D(J)= DFOUR(F(J),N,T,X)
        ENDDO
C        TYPE *,'Computing the spectral window...'
        DO J= 0,2*M
           W(J)= DFOUR(F(J),N,T,ONES)
        ENDDO

C  create the headers & write to the output files
C    ---dirty spectrum
c        WRITE(HEADER,'(A10,'',from:'',A10,'',Ndat='',I5,'',Tsub='',
c     &   1P,E14.7,'',Dsub='',1P,E14.7)') DFILE,XFILE,N,TMEAN,XMEAN
c        CALL WRSPEC(M,F,D,DFILE,HEADER)
C        TYPE *,HEADER
C    ---spectral window
c        WRITE(HEADER,'(A10,'',from:'',A10,'',Ndat='',I5,'',Tsub='',
c     &   1P,E14.7,'',Tavg='',1P,E14.7)') WFILE,XFILE,N,TMEAN,TMEAN
c        CALL WRSPEC(2*M,F,W,WFILE,HEADER)
C        TYPE *,HEADER

C  exit the program

        OPEN(UNIT=1,FILE='dirty.dat',STATUS='UNKNOWN')
         J=0
         PMAXOLD=0.0
C        write the data to the screen
         DO 22, J=0,M
         POWER(J)=SQRT(REAL(D(J))*REAL(D(J))+AIMAG(D(J))*AIMAG(D(J)))
         IF(POWER(J).GT.PMAX) THEN 
             PMAX=POWER(J)
             FMAX=F(J)
             K=J
         ENDIF 
         WRITE(1,'(3(1X,1P,E14.7))') F(J),POWER(J)
22      CONTINUE
	  close(1)

	  CALL CLEAN(M,F,D,W)





        CALL EXIT
 100  Format(a,\)
        END









        COMPLEX FUNCTION DFOUR(FREQ,N,TIME,DATA)
C----------------------------------------------------------------------------
C  Author:   J. Lehar                                         Date: 13-JUL-87
C----------------------------------------------------------------------------
C  Returns the Fourier transform of the time series specified by TIME(1:N)
C  and DATA(1:N), evaluated at the frequency FREQ.
C  The form of the Fourier transform is taken from Bracewell (1965)
C
C                     1  .--. N           -i*2*PI*F*TIME(i)
C           DFT(F) = ---  >      DATA(i) e
C                     N  `--'i=1
C
C  The DFT is normalized to have the data mean at FREQ=0.
C----------------------------------------------------------------------------
        COMPLEX    SUM
        REAL       DATA(N),TIME(N)

C  initialize some variables
        SUM= (0.,0.)                    ! accumulation variable
        TWOPI= 8.*ATAN2(1.,1.)          ! calculate 2*pi

C  Evaluate FT at F...
        DO I=1,N
           PHASE= -TWOPI*FREQ*TIME(I)
           SUM= SUM + DATA(I)*CMPLX(COS(PHASE),SIN(PHASE))
        ENDDO

C  return with FT properly normalized
        DFOUR= SUM/N           ! ensures correct normalization
        RETURN
        END





        SUBROUTINE RDDATA(N,TIME,DATA,DFILE,HEADER)
C----------------------------------------------------------------------------
C  Author:  J. Lehar                                         Date:  13-JUL-87
C  Written for VAX FORTRAN 77   (May require some changes for other machines)
C  Source file:  READWRITE.FOR
C----------------------------------------------------------------------------
C  Reads in the times, data values for a time series from DFILE
C  In addition, a descriptive header is read in.  N (input) specifies the
C  maximum number of samples allowed and returns (output) the number of
C  samples found.  OPEN uses UNIT=1
C     ARGUMENTS:
C         N          (input) #samples allowed, (output) #samples found
C         TIME       (output) array for time samples
C         DATA       (output) array for data samples
C         DFILE      (input) data file name
C         HEADER     (output) descriptive file header
C----------------------------------------------------------------------------
        REAL TIME(N),DATA(N)
        CHARACTER DFILE*(*),HEADER*80

C  open the file & read in the header
        OPEN(UNIT=1,FILE=DFILE,STATUS='OLD')
C        READ(1,'(A80)') HEADER

C  read in the time series, each record has (Time,Data)
        I=1
C       step through the file until END-OF-FILE
c10         READ(1,'(2(1X,1P,E14.9))',END=11) TIME(I),DATA(I)
c10         READ(1,'(2(1X,E14.10))',END=11) TIME(I),DATA(I)
10          READ(1,*,END=11) TIME(I),DATA(I)
c           IF (TIME(I).EQ.TIME(I-1)) TYPE *, 'HERE', I, TIME(I)
c            IF (TIME(I).lt.9400.or.TIME(I).gt.9875) goto 10
C           TYPE *, TIME(I), DATA(I)
           I= I+1
           IF(I.GT.N)THEN
C              TYPE *,'RDDATA: data exceeds array size, truncating at N=',N
              GOTO 11
           ENDIF
           GOTO 10
11      CONTINUE
C        type *, 'here'
C  close the file & return the number of samples found
        CLOSE(UNIT=1)
        N= I-1
        RETURN
        END









        SUBROUTINE WRDATA(N,TIME,DATA,DFILE,HEADER)
C----------------------------------------------------------------------------
C  Author:  J. Lehar                                         Date:  13-JUL-87
C  Written for VAX FORTRAN 77   (May require some changes for other machines)
C  Source file:  READWRITE.FOR
C----------------------------------------------------------------------------
C  Writes the times, data values for a time series from DFILE
C  In addition, a descriptive header is written. OPEN uses UNIT=1
C     ARGUMENTS:
C         N          (input) #samples
C         TIME       (input) array for time samples
C         DATA       (input) array for data samples
C         DFILE      (input) data file name
C         HEADER     (input) descriptive file header
C----------------------------------------------------------------------------
        REAL TIME(N),DATA(N)
        CHARACTER DFILE*(*),HEADER*80

C  open the file & write the header
        OPEN(UNIT=1,FILE=DFILE,STATUS='UNKNOWN')
        WRITE(1,'(A80)') HEADER

C  write the time series
        DO I=1,N
           WRITE(1,'(2(1X,1P,E14.7))') TIME(I),DATA(I)
        ENDDO

C  close the file & return
        CLOSE(UNIT=1)
        RETURN
        END









        SUBROUTINE RDSPEC(M,FREQ,SPEC,SFILE,HEADER)
C----------------------------------------------------------------------------
C  Author:  J. Lehar                                         Date:  13-JUL-87
C  Written for VAX FORTRAN 77   (May require some changes for other machines)
C  Source file:  READWRITE.FOR
C----------------------------------------------------------------------------
C  Reads in the frequency and spectral values for a spectrum to SFILE.
C  In addition, a descriptive header is read in.  M (input) specifies the
C  maximum frequency element allowed and returns (output) the maximum
C  frequency element found.  OPEN uses UNIT=1
C     ARGUMENTS:
C         M       (input) max. freq element allowed, (output) max. elt. found
C         FREQ    (output) array for frequency samples
C         SPEC    (output) array for spectrum samples
C         SFILE   (input) spectrum file name
C         HEADER  (output) descriptive file header
C----------------------------------------------------------------------------
        COMPLEX SPEC(0:M)
        REAL FREQ(0:M)
        CHARACTER SFILE*(*),HEADER*80

C  open the file & read in the header
        OPEN(UNIT=1,FILE=SFILE,STATUS='OLD')
        READ(1,'(A80)') HEADER

C  read in the spectrum, each record has (Freq,Real.comp,Imag.comp)
        I=0
C       step through the file until END-OF-FILE
10         READ(1,'(3(1X,1P,E14.7))',END=11) FREQ(I),SPEC(I)
           I= I+1
           IF(I.GT.M)THEN
C              TYPE *,'RDSPEC: data exceeds array size, truncating at M=',M
              GOTO 11
           ENDIF
           GOTO 10
11      CONTINUE

C  close the file & return the max. frequency element found
        CLOSE(UNIT=1)
        M= I-1
        RETURN
        END









        SUBROUTINE WRSPEC(M,FREQ,SPEC,SFILE,HEADER)
C----------------------------------------------------------------------------
C  Author:  J. Lehar                                         Date:  13-JUL-87
C  Written for VAX FORTRAN 77   (May require some changes for other machines)
C  Source file:  READWRITE.FOR
C----------------------------------------------------------------------------
C  Writes the frequency and spectral values for a spectrum to SFILE
C  In addition, a descriptive header is written. OPEN uses UNIT=1
C     ARGUMENTS:
C         M       (input) max. frequency element
C         FREQ    (input) array for frequency samples
C         SPEC    (input) array for spectrum samples
C         SFILE   (input) spectrum file name
C         HEADER  (input) descriptive file header
C----------------------------------------------------------------------------
        COMPLEX SPEC(0:M)
        REAL FREQ(0:M)
        CHARACTER SFILE*(*),HEADER*80

C  open the file & read in the header
        OPEN(UNIT=1,FILE=SFILE,STATUS='UNKNOWN')
        WRITE(1,'(A80)') HEADER

C  write out the spectrum, the spec. file lists FREQ,REAL(SPEC),IMAG(SPEC)
        DO I=0,M
           WRITE(1,'(3(1X,1P,E14.7))') FREQ(I),SPEC(I)
        ENDDO

C  close the file & return
        CLOSE(UNIT=1)
        RETURN
   	END




C*****************************************************
C***  Library: KUSS_FORTRAN
C***  Module: CLEAN
C*****************************************************
        SUBROUTINE CLEAN(M,F,R,W)
C----------------------------------------------------------------------------
C  Author:   J. Lehar                                         Date: 13-JUL-87
C  Written for VAX FORTRAN 77   (May require some changes for other machines)
C  NOTE: This code was written for clarity, not for efficiency.
C        Some optimization may be desired for certain applications.
C----------------------------------------------------------------------------
C  Deconvolves the spectral window in WFILE from the dirty spectrum in DFILE,
C  by using an iterative, one-dimensional, complex, Hoegbom CLEAN algorithm.
C  The resulting clean spectrum is in SFILE, the residual and CLEAN component
C  spectra are found in RFILE & CFILE, respectively.
C     The user determines the gain and the number of cleans, the latter
C  either explicitly or by specifying a CLEAN level.  (when the maximum in
C  the residual spectrum is less than the CLEAN level, cleaning stops.)
C    Since all spectra from real data are Hermitian, we define only the non-
C  negative frequencies.  The negative frequency elements are recovered by
C  the use of the function CVAL, which returns the complex conjugate for
C  negative frequencies, and zero for frequencies outside the defined range.
C  NOTE: the filenames are limited to 10 characters.  This is so they can
C        be propagated to subsequent programs through the spec. file headers
C----------------------------------------------------------------------------
C  CALLS:     RDSPEC               to read spectra   \_ in READWRITE.FOR
C             WRSPEC               to write spectra  /
C             CLEAN1               performs 1 iteration of CLEAN
C             FITBEAM              to fit a beam to the spectral window
C             RESTORE              to create the clean spectrum
C----------------------------------------------------------------------------
        PARAMETER (MAXM= 2000000)      ! maximum index for spectral arrays
c        PARAMETER (MAXM= 800000)      ! maximum index for spectral arrays

C  Declare arrays and some variables
        COMPLEX   W(0:2*MAXM),B(0:MAXM),R(0:MAXM),C(0:MAXM),S(0:MAXM)
        REAL*4    F(0:2*MAXM), POWER(0:MAXM)
        CHARACTER WFILE*20,DFILE*20,RFILE*20,CFILE*20,SFILE*20,HEADER*80
        CHARACTER XFILE*20

C  get execution parameters & filenames
c        WRITE(*,*) 'INPUT FILES:'
c        WRITE(*,*) '(''$Dirty spectrum file: '')'
c        READ (*,*)        DFILE
         dfile='d'
c        WRITE(*,*) '(''$Spectral window file: '')'
c        READ (*,*) WFILE
         wfile='w'
c        WRITE(*,*) 'INPUT PARAMETERS:'
        WRITE(*,100) ' Gain: '
        READ (*,*) GAIN
        WRITE(*,100) ' Cleans ( >0 = #CLEANs, <0 = CLEAN level ): '
        READ (*,*) CLEANS
c        WRITE(*,*) 'OUTPUT FILES:'
c        WRITE(*,*) '(''$Residual spectrum file: '')'
c        READ (*,*) RFILE
c        rfile='r'
c        WRITE(*,*) '(''$CLEAN components file: '')'
c        READ (*,*) CFILE
c        cfile='c'
c        WRITE(*,*) '(''$Clean spectrum file: '')'
c        READ (*,*) SFILE
c        sfile='s.dat'
C    ---determine depth of CLEAN
        IF(CLEANS.GE.0.)THEN
           NCLEAN= INT(CLEANS)   ! clean to the specified # of cleans
           RCLEAN= 0.            ! no level specified.
        ELSE
           NCLEAN= 32000.        ! more than anyone is likely to want
           RCLEAN= ABS(CLEANS)   ! specifies the desired level
        ENDIF

C  read the input spectra, get time info for beam, determine freq. range.
C    ---dirty spectrum, read into the residual R(0:MS)
        MS= MAXM                         ! max. allowed MS, for RDSPEC


      goto 555

        CALL RDSPEC(MS,F,R,DFILE,HEADER)
C        TYPE *,HEADER
C    ---spectral window W(0:MW)
        MW= 2*MAXM                       ! max. allowed MW, for RDSPEC
        CALL RDSPEC(MW,F,W,WFILE,HEADER)
C        TYPE *,HEADER


555	CONTINUE
	MS=M
	MW=M*2

c        READ(HEADER,'(43X,E14.7,6X,E14.7)') TZERO,TMEAN  ! from WFILE
        dF= F(1)-F(0)                    ! frequency increment
        IF(2*MS.GT.MW) MS= MW/2          ! ensure that MS < MW/2

C  CLEAN the residual spectrum, storing the components in C(0:MS)
C        TYPE *,'CLEANing up to ',NCLEAN,'  iterations, or down to ',RCLEAN
        DO 1 K=1,NCLEAN                        ! on until NCLEAN
           CALL CLEAN1(MW,W,MS,R,C,GAIN,RMAX)  ! execute a single CLEAN
           IF(RMAX.LT.RCLEAN) GOTO 10          ! skip out if down to RCLEAN
 1	continue
10      NCLEAN= K-1                            ! actual number of CLEANs
C        TYPE *,'NCLEAN=',NCLEAN,'           RMAX=',RMAX

C  Generate the beam B(0:MB) and the clean spectrum S(0:MS)
        MB= MS                          ! max. possible MB, for FITBEAM
        CALL FITBEAM(MB,B,MW,W,dF,TZERO,TMEAN)  ! create the beam
        CALL RESTORE(MS,S,C,R,MB,B)     ! restore S (convolve CxB, add R)

C  Write the output spectra
C    ---residual spectrum
c        WRITE(HEADER,1000) RFILE,DFILE,WFILE,NCLEAN,GAIN
c        CALL WRSPEC(MS,F,R,RFILE,HEADER)
C        TYPE *,HEADER
C    ---CLEAN components
c        WRITE(HEADER,1000) CFILE,DFILE,WFILE,NCLEAN,GAIN
c        CALL WRSPEC(MS,F,C,CFILE,HEADER)
C        TYPE *,HEADER
C    ---clean spectrum
c        WRITE(HEADER,1000) SFILE,DFILE,WFILE,NCLEAN,GAIN
c        CALL WRSPEC(MS,F,S,SFILE,HEADER)
C        TYPE *,HEADER
C     - format statement for spectrum file headers...
1000    FORMAT(' FILE: ',A10,' FROM ',A10,' AND ',A10,';  #CYCLES:',
     &         I5,', GAIN:',F6.4)

C exit the program


cC  open the file & read in the header
c        OPEN(UNIT=1,FILE=XFILE,STATUS='OLD')
c        READ(1,'(A80)') HEADER
cc        TYPE *, HEADER

        OPEN(UNIT=1,FILE='clean.dat',STATUS='UNKNOWN')

         J=0
         PMAXOLD=0.0
C        write the data to the screen
         DO 22, J=0,MS
         POWER(J)=SQRT(REAL(S(J))*REAL(S(J))+AIMAG(S(J))*AIMAG(S(J)))
         IF(POWER(J).GT.PMAX) THEN 
             PMAX=POWER(J)
             FMAX=F(J)
             K=J
         ENDIF 
         WRITE(1,'(3(1X,1P,E14.7))') F(J),POWER(J)
22      CONTINUE
	  close(1)

C         WRITE(*,*) K, FMAX, PMAX
 100  Format(a,\)



        END









        SUBROUTINE CLEAN1(MW,WIN,MS,RES,CMP,GAIN,RMAX)
C----------------------------------------------------------------------------
C  Author:   J. Lehar                                         Date: 13-JUL-87
C----------------------------------------------------------------------------
C  Performs one iteration of the RLD complex, 1-D, CLEAN algorithm.
C  o  The component, CCPOS, responsible for the max., at L, in the residual
C     spectrum RES, is estimated, taking into account the interference with
C     its conjugate, CCNEG at -L.   ( through RES(L)= CCPOS +CCNEG*WIN(2L) )
C  o  The spectral window WIN is matched to these components, multiplied by
C     a GAIN factor and subtracted from the residual spectrum.
C  o  GAIN*CCPOS is added to the CMP spectrum.
C  o  returns RMAX, the max. in the residual (absolute value)
C----------------------------------------------------------------------------
C  CALLS:     CVAL                 determines spectral values beyond (0:M)
C----------------------------------------------------------------------------
        COMPLEX WIN(0:MW),RES(0:MS),CMP(0:MS),WIN2L,CCPOS,CCNEG,CVAL
        DATA ERROR/0.0001/                     ! allowed error in WNORM

C  find the location L of max. in the residual spectrum RES(0:M)
        RMAX= CABS(RES(0))
        L= 0
        DO 2 J=1,MS
           IF(CABS(RES(J)).GT.RMAX)THEN
              RMAX= CABS(RES(J))
              L= J
           ENDIF
 2      continue 

C  estimate the component at L which yields RES(L) through WIN(L)
        WIN2L= WIN(2*L)                       ! (L,-L) interference
        WNORM= 1.0-CABS(WIN2L)**2
        IF(WNORM.LT.ERROR) CCPOS= 0.5*RES(L)  ! prevent singularities
        IF(WNORM.GE.ERROR) CCPOS= ( RES(L)-WIN2L*CVAL(RES,-L,MS) )/WNORM

C  Remove effects of both +L and -L components from RES, add CCPOS to CMP
        CCPOS= GAIN*CCPOS                     ! remove GAIN*CCPOS
        CCNEG= CONJG(CCPOS)                   ! conjugate at -L
        DO 3 J=0,MS
           RES(J)= RES(J) -CCPOS*CVAL(WIN,J-L,MW) -CCNEG*WIN(J+L)
 3      continue     !              (from L )          (from -L )
        CMP(L)= CMP(L) + CCPOS                ! add it to CMP

C  return to the calling program
        RETURN
        END









        SUBROUTINE FITBEAM(MB,BEAM,MW,WIND,dF,TZERO,TMEAN)
C----------------------------------------------------------------------------
C  Author:   J. Lehar                                         Date: 13-JUL-87
C----------------------------------------------------------------------------
C  Fits a clean beam, BEAM, to the primary peak of the spectral window, WIND.
C  The modulus is a Gaussian, matched to have the same half-width,half-max.
C  through linear interpolation between frequency samples.
C  The phase is a linear function of frequency, determined by the frequency
C  increment dF and the time offset from the mean time (TZERO-TMEAN).
C  NOTE:  DFOURT subtracts the time average, so that TZERO=TMEAN always.
C         therefore, beams for files from DFOURT will always be purely real.
C         If two spectra from different time series are to be added, DFOURT
C         must be modified to allow a fixed time to be subtracted from both
C         time series. Then the phases in the spectra and the FITBEAM phases
C         will be correctly constructed for combination of the spectra.
C----------------------------------------------------------------------------
        COMPLEX BEAM(0:MB),WIND(0:MW)

C  Find the half-width,half-max of CABS(WIND)
        HALFMAX= 0.5*CABS(WIND(0))         ! Max. of WIND at J=0
        HALFWID= 0.                        !<---(not yet found)
        DO 4 J=1,MB                         ! step along WIND
           IF(CABS(WIND(J)).LT.HALFMAX)THEN              !\   When .LT.HALFMAX,
              WCURR= CABS(WIND(J))                       ! \  use lin. inter-
              WLAST= CABS(WIND(J-1))                     !  > polation to find
              SLOPE= 1./(WCURR-WLAST)                    ! /  the half-width
              HALFWID= FLOAT(J-1) + SLOPE*(HALFMAX-WLAST)!/
              GOTO 10                                    ! then skip out
           ENDIF
 4      continue                             ! keep on until up to MB
10      continue 
C TYPE *,'*Could not find HALF-WIDTH,HALF-MAX*'
C
C  match a Gaussian of width SIGMA to the above, determine phase increment
        TWOPI= 8.*ATAN2(1.,1.)             ! calculate 2*PI to precision
        SIGMA= HALFWID/SQRT(2.*ALOG(2.))   !\ parameters for Gaussian
        CONST= 1./(2.*SIGMA*SIGMA)         !/
        dPHASE= TWOPI*dF*(TZERO-TMEAN)     ! dPHASE from time info.

C  Fill the BEAM array
        MB= INT(5.*SIGMA) +1               ! only fill out to 5 sigmas
        DO 5 J= 0,MB
           X= FLOAT(J)
           GAUSS= EXP(-CONST*X*X)          ! Gaussian envelope
           PHASE= dPHASE*X                 ! phase linear in F
           BEAM(J)= GAUSS*CMPLX(COS(PHASE),SIN(PHASE))
  5     continue

C  return to the calling program
        RETURN
        END









        SUBROUTINE RESTORE(MS,SP,CC,RS,MB,BM)
C----------------------------------------------------------------------------
C  Author:   J. Lehar                                         Date: 13-JUL-87
C----------------------------------------------------------------------------
C  Creates the clean spectrum SP(0:MS), by convolving the CLEAN
C  components CC(0:MS) with the clean beam BM(0:MB), and finally
C  adding the residual spectrum RS(0:MS) to the result.
C     This produces a spectrum estimate which more properly represents
C  the frequency resolution (through the beam width) and which includes
C  the fluctuations in the residual spectrum.  The CLEAN spectrum will
C  also combine components within one beam-width of each other to build
C  up a signal which might appear as several smaller ones in CC.  These
C  should be combined, because frequency separations of less than one
C  beam width are not reliable.
C----------------------------------------------------------------------------
C  CALLS:     CVAL                 determines spectral values beyond (0:M)
C----------------------------------------------------------------------------
        COMPLEX SP(0:MS),CC(0:MS),RS(0:MS),BM(0:MB),CVAL

C  Convolve CC with BM, then add RS, to form SP
        DO 6 J=0,MS
           SP(J)= (0.,0.)    ! reset SP(J)
           DO 7 K=-MB,MB
              SP(J)= SP(J)+CVAL(BM,K,MB)*CVAL(CC,J-K,MS)  ! convolution
 7         continue
           SP(J)= SP(J)+RS(J)
 6      continue

C  return to the calling program
        RETURN
        END




        COMPLEX FUNCTION CVAL(SPEC,L,M)
C----------------------------------------------------------------------------
C  Author:   J. Lehar                                         Date: 13-JUL-87
C----------------------------------------------------------------------------
C  Returns the "value" of the spectrum array SPEC(0:M) at the "location" L.
C     if  L  >= 0    CVAL returns SPEC(L)
C     if  L  >  0    CVAL returns the complex conjugate of SPEC(L)
C     if |L| >  M    CVAL will return (0.,0.)
C----------------------------------------------------------------------------
        COMPLEX SPEC(0:M)

C  set value to conjugate of absolute location if necessary
        LOC= ABS(L)               ! where necessery information is found
        IF(LOC.GT.M)THEN                    !\
           CVAL= (0.,0.)                    ! \   if LOC exceeds range
           RETURN                           ! /   return (0.,0.)
        ENDIF                               !/
        IF(L.GE.0) CVAL= SPEC(LOC)          ! return SPEC(L)
        IF(L.LT.0) CVAL= CONJG(SPEC(LOC))   ! return SPEC(-L)
        RETURN
        END

